/*  $Id: menu.h,v 1.1 2003/11/02 11:36:43 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

//! clanGUI="Controls"
//! header=gui.h

#ifndef header_menu
#define header_menu

#if _MSC_VER > 1000
#pragma once
#endif

#include "component.h"
#include "../Core/Math/point.h"

class CL_MenuNode;
class CL_Menu_Generic;

//: Menu component
class CL_Menu : public CL_Component
{
//! Construction:
 public:
	//: CL_Menu Constructor
	CL_Menu(
		const CL_Point &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL,
		bool vertical=false);

	//: CL_Menu Constructor
	CL_Menu(
		CL_MenuNode *parent_node,
		CL_Component *parent,
		CL_StyleManager *style = NULL );
	
	//: CL_Menu Destructor
	virtual ~CL_Menu();

//! Attributes:
 public:
	//: Returns true if the menu is open
	bool is_open();

	//: Returns true if the menu is vertical, false if horizontal
	bool is_vertical();

	//: Returns true if the mouse cursor is on top of the menu
	bool has_mouse_over();

	//: Returns true if the mouse cursor is in a open submenu of the menu
	bool has_mouse_in_submenus();

	//: Returns the parent node of this menu.
	CL_MenuNode *get_parent_node();
	
	//: Returns the top level parent of the menu tree.
	CL_Menu *get_root_menu();

	//: Returns a list containing the CL_MenuNode's in this menu.
	std::list<CL_MenuNode*> &get_node_list();

	//: Returns the total height of items in a menu
	int get_items_height();

	//: Returns the total width of items in a menu
	int get_items_width();

	//: Returns true if the menus submenus need to be clicked to open.
	//: By default they open on mouse enter for vertical menus and
	//: on click with horizontal menus when it doesn't have focus.
	bool is_click_to_open();

//! Signals:
 public:
	//: This signal is emitted when a repositioning of the menus is requsted.
	//param int x : x position of topleft of the menu
	//param int y : y position of topleft of the menu
	CL_Signal_v2<int,int> &sig_reposition();
	
//! Operations:
 public:
	//: Calculates a position and size for the menu, using code in the GUI theme.
	void reposition();

	//: Calculates a position and size for the menu, using code in the GUI theme.
	//param int x : x position of topleft of the menu
	//param int y : y position of topleft of the menu	
	void reposition(int,int);

	//: Makes the menu vertical (true) or horizontal (false).
	void set_vertical(bool vertical);

	//: Add a CL_MenuNode to the menu.
	void add_node(CL_MenuNode *node);
	
	//: Remove a CL_MenuNode from the menu.
	void remove_node(CL_MenuNode *node);

	//: Set the parent node of this menu.
	void set_parent_node(CL_MenuNode *);
	
	//: Open the menu.
	void open();

	//: Close the menu.
	void collapse();

	//: Close all submenus of the menu.
	void collapse_submenus();

	//: Set the opening mode of submenus to open on click (true) or open on mouse over (false)
	void set_click_to_open(bool click);

//! Implementation:
 private:
	CL_Menu(const CL_Menu &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	
	CL_Menu_Generic *impl;

	friend class CL_MenuNode_Generic;
};

#endif
