/*  $Id: menu_generic.cpp,v 1.3 2003/11/02 15:07:49 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "API/gui.h"
#include "API/signals.h"
#include <iostream>
#include "menu_generic.h"
#include "API/GUI/menu.h"
#include "API/GUI/menu_node.h"
#include "API/Display/mouse.h"

CL_Menu_Generic::CL_Menu_Generic(
	CL_Menu *self,
	bool vertical,
	CL_MenuNode *parent_node)
:
	parent_node(parent_node),
	vertical(vertical),
	menu_open(true),
	submenu_open(false),
	click_to_open(false),
	close_lock(false),
	menu(self)
{
	if( parent_node == 0 )
	{
		slots.connect( CL_Mouse::sig_key_up(), this, &CL_Menu_Generic::on_mouse_up);
//		slots.connect( CL_Mouse::sig_key_down(), this, &CL_Menu_Generic::on_mouse_up);
		click_to_open = true;
	}

	if( vertical ) collapse();

//	menu->set_topmost_flag(true);
//	menu->raise();
}

CL_Menu_Generic::~CL_Menu_Generic()
{
	// todo: delete?

	std::list<CL_MenuNode *>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		delete (*it);
	}
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

CL_Menu *CL_Menu_Generic::get_root_menu()
{
	CL_Menu *test_parent = menu;
	
	while( test_parent != NULL )
	{
		if( test_parent->get_parent_node() == NULL )
			return test_parent;
		else
			test_parent = test_parent->get_parent_node()->get_parent_menu();
	}

	return NULL;
}

int CL_Menu_Generic::get_items_width()
{
	int width = 0;

	std::list<CL_MenuNode*>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		int child_width = (*it)->get_data()->get_width();

		if( vertical )
		{
			if( child_width > width )
				width = child_width;
		}
		else
		{
			width += child_width;
		}

	}
	
	return width;
}

int CL_Menu_Generic::get_items_height()
{
	int height = 0;
	
	if( !children.empty() )
	{
		std::list<CL_MenuNode*>::iterator it;
		for( it = children.begin(); it != children.end(); ++it )
		{
			height += (*it)->get_data()->get_height() + 3;
		}
	}
	
	return height;
}

bool CL_Menu_Generic::has_mouse_in_submenus()
{
	if( !menu_open ) 
		return false;
	
	std::list<CL_MenuNode*>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		if( (*it)->has_mouse_in_submenus() )
			return true;
	}
	
	return false;
}

bool CL_Menu_Generic::has_mouse_over()
{
	if( !menu_open ) 
		return false;
	
	std::list<CL_MenuNode*>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		if( (*it)->has_mouse_over() )
			return true;
	
		if( (*it)->get_data() )
			if( (*it)->get_data()->has_mouse_over() )
				return true;
	}
	
	return false;
}


/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_Menu_Generic::add_node(CL_MenuNode *node)
{
	node->set_parent_menu(menu);	
	children.push_back(node);
}

void CL_Menu_Generic::remove_node(CL_MenuNode *node)
{
	std::list<CL_MenuNode *>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		if( (*it) == node )
			it = children.erase(it);
	}
	
	CL_Menu *root = get_root_menu();
	CL_Rect pos = root->get_position();
	root->reposition(pos.left, pos.top);
}

void CL_Menu_Generic::open()
{
	menu->show(true);
	menu->enable(true);
	menu_open = true;
}

void CL_Menu_Generic::collapse()
{
	collapse_submenus();

	menu->show(false);
	menu->enable(false);
	menu_open = false;
}

void CL_Menu_Generic::collapse_submenus()
{
	std::list<CL_MenuNode*>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		(*it)->collapse();
	}
}	


/////////////////////////////////////////////////////////////////////////////
// Callbacks:

// on_mouse_up() is connected only to the root menu. It will close
// the menu if a click outside the menus takes place.
void CL_Menu_Generic::on_mouse_up(const CL_InputEvent&)
{
	if( close_lock ) // prevent close immediately after opening.
	{
		close_lock = false;
		return;
	}

/*	bool mouse_in_self = false;	

	if( menu->has_mouse_over() )
		mouse_in_self = true;
	else
	{
		std::list<CL_MenuNode*>::iterator it;
		for( it = children.begin(); it != children.end(); ++it )
		{
			if( (*it)->has_mouse_over() )
				mouse_in_self = true;

			CL_Component *data = (*it)->get_data();

			if( data )
				if( data->has_mouse_over() )
					mouse_in_self = true;
		}	
	}
*/

/*	bool mouse_in_submenus = false;

	std::list<CL_MenuNode*>::iterator it;
	for( it = children.begin(); it != children.end(); ++it )
	{
		if( (*it)->has_mouse_in_submenus() )
		{
			mouse_in_submenus = true;
			break;
		}
	}
*/
//	if( /*!mouse_in_self &&*/ !has_mouse_in_submenus() )
	if( !menu->has_mouse_in_submenus() )
	{
		collapse_submenus();
		click_to_open = true;
	}
}
